import os, sys, argparse
from . import get_data
from .utils import *

def main():
    USAGE_MSG = 'snapmap_archiver -o [OUTPUT DIR] -l="[LATITUDE],[LONGITUDE]"'
    geo_msg = 'Use comma seperated values for latitude/longitude, e.g: -l="35.0,67.0"'
    parser = argparse.ArgumentParser(description='Download content from Snapmaps', usage=USAGE_MSG)
    parser.add_argument('-o', dest='output_dir', type=str, help='Output directory for downloaded content.')
    parser.add_argument('-l', '--location', dest='location', type=str, help='Latitude/longitude of desired area.')
    parser.add_argument('-z', dest='zoom_depth', type=float, help='Snapmaps zoom depth, default is 5.')
    parser.add_argument('-r', dest='radius', type=int, help='Maximum Snap radius in meters, default is 10000.')
    parser.add_argument('--write-json', dest='write_json', action='store_true', default=False, help='Write Snap metadata JSON.')
    parser.add_argument('--use-ffmpeg', dest='use_ffmpeg', action='store_true', default=False, help='Use ffmpeg to merge graphical elements to video Snaps.')
    args = parser.parse_args()

    if(not args.output_dir):
        print('Output directory (-o) is required.')
        sys.exit(USAGE_MSG)

    if(not args.location):
        print('location (-l) is required.')
        sys.exit(USAGE_MSG)

    if(',' not in args.location):
        sys.exit(geo_msg)

    if(not os.path.isdir(args.output_dir)):
        try:
            os.mkdir(args.output_dir)
        except:
            sys.exit(f'Could not create directory "{args.output_dir}"')

    if(not args.radius):
        args.radius = 10000
    elif(args.radius > 85000):
        print('Supplied radius value is too large (above 85,000). Defaulting to 85000.')
        args.radius = 85000
    
    try:
        geo_data = args.location.split(',', 1)
    except:
        sys.exit(geo_msg)
    api_response = get_data.api_query(float(geo_data[0]), float(geo_data[1]), max_radius=args.radius)
    download_media(args.output_dir, organise_media(api_response), args.write_json)
    



import sys, json, subprocess, os

def organise_media(api_data):
    to_download = []
    print(f'Found {len(api_data)} Snaps.')
    for entry in api_data:
        data_dict = {'id': entry['id'], 'create_time': entry['timestamp'], 'media': {}}
        for locale in entry['snapInfo']['title']['strings']:
            if(locale['locale'] == 'en'):
                data_dict['location'] = locale['text']
        try:
            data_dict['media']['raw_url'] = entry['snapInfo']['streamingMediaInfo']['prefixUrl'] + 'media.mp4'
            try:
                data_dict['media']['video_overlay'] = entry['snapInfo']['streamingMediaInfo']['prefixUrl'] + 'overlay.png'
            except:
                data_dict['media']['video_overlay'] = None
        except:
            try:
                data_dict['media']['raw_url'] = entry['snapInfo']['publicMediaInfo']['publicImageMediaInfo']['mediaUrl']
            except:
                for i in entry['snapInfo'].items():
                    if(i[0] == 'streamingThumbnailInfo'): # For some reason JSON throws an error if you just query this key directly, so you have to do it this way.
                        data_dict['media']['raw_url'] = i[1]['infos'][-1]['thumbnailUrl']
                if(len(data_dict['media']) == 0):
                    continue # If there's no video file and no video/image thumbnail, just skip the snap since there's nothing to download
        to_download.append(data_dict)
    return(to_download)

def download_media(output_dir, organised_data, dl_json=False):
    for index,snap in enumerate(organised_data):
        DL_MSG = f'Snap {index + 1}/{len(organised_data)} downloading...'

        filename = snap['location'] + ' - ' + snap['create_time'] + ' - ' + snap['id']
        if(dl_json):
            with open(f'{output_dir}/' + filename + '.info.json', 'w') as json_file:
                json_file.write(json.dumps(snap, indent=2))
        if(sys.platform == 'win32'):
            cmd = ['aria2c.exe', snap['media']['raw_url'], '-d', output_dir, '-o']
        else:
            cmd = ['aria2c', snap['media']['raw_url'], '-d', output_dir, '-o']
        
        if(snap['media']['raw_url'][-3:] == 'mp4'):
            if(os.path.exists(f'{cmd[-2]}/' + filename + '.mp4')):
                print(f'Snap {index + 1}/{len(organised_data)} already downloaded.')
            else:
                print(DL_MSG + f' - {filename}.mp4')
                subprocess.run(cmd + [filename + '.mp4'], capture_output=True)
        else:
            if(os.path.exists(f'{cmd[-2]}/' + filename + '.jpg')):
                print(f'Snap {index + 1}/{len(organised_data)} already downloaded.')
            else:
                print(DL_MSG + f' - {filename}.jpg')
                subprocess.run(cmd + [filename + '.jpg'], capture_output=True)


import requests, random, json, pathlib, sys

from requests.api import get

req_headers = {
    'User-Agent': random.choice(json.loads(open(f'{pathlib.Path(__file__).parent.absolute()}/utils/user-agents.json', 'r').read())),
    'Host': 'ms.sc-jpl.com',
    'Accept': '*/*',
    'Accept-Language': 'en-US',
    'Accept-Encoding': 'gzip, deflate, br',
    'Referer': 'https://map.snapchat.com/@0,0,7.67z',
    'Content-Type': 'application/json',
    'Origin': 'https://map.snapchat.com',
    'Content-Length': '2',
    'Connection': 'keep-alive',
    'DNT': '1',
    'Sec-GPC': '1',
    'TE': 'Trailers',
}

def get_epoch():
    for entry in json.loads(requests.post('https://ms.sc-jpl.com/web/getLatestTileSet', headers=req_headers, json={}).text)['tileSetInfos']:
        if(entry['id']['type'] == 'HEAT'):
            return(entry['id']['epoch'])

def api_query(lat, lon, zl=5, max_radius=10000):
    available_snaps = []
    current_iteration = max_radius
    _epoch = get_epoch()
    try:
        print('Querying Snaps...')
        while current_iteration != 1:
            payload = {"requestGeoPoint":{"lat":lat,"lon":lon},"zoomLevel":zl,"tileSetId":{"flavor":"default","epoch":_epoch,"type":1},"radiusMeters":current_iteration,"maximumFuzzRadius":0}
            req_headers['Content-Length'] = str(len(str(payload)))
            api_data = json.loads(requests.post('https://ms.sc-jpl.com/web/getPlaylist', headers=req_headers, json=payload).text)
            available_snaps = available_snaps + api_data['manifest']['elements']
            if(current_iteration > 2000):
                current_iteration = current_iteration - 2000
            elif(current_iteration > 1000):
                current_iteration = current_iteration - 100
            else:
                current_iteration = 1
        return [i for n, i in enumerate(available_snaps) if i not in available_snaps[n + 1:]]
    except:
        sys.exit("You seem to have been rate limited, please wait and try again.")